@@ -1,19 +1,24 @@
 from __future__ import unicode_literals
 
 import os
-import mock
+import shutil
+import sys
 import tempfile
 import unittest
 import pkg_resources
 
-import six
-
 from fs import open_fs, opener
 from fs.osfs import OSFS
 from fs.opener import registry, errors
 from fs.memoryfs import MemoryFS
 from fs.appfs import UserDataFS
 from fs.opener.parse import ParseResult
+from fs.opener.registry import Registry
+
+try:
+    from unittest import mock
+except ImportError:
+    import mock
 
 
 class TestParse(unittest.TestCase):
@@ -23,7 +28,7 @@ class TestParse(unittest.TestCase):
 
     def test_parse_not_url(self):
         with self.assertRaises(errors.ParseError):
-            parsed = opener.parse("foo/bar")
+            opener.parse("foo/bar")
 
     def test_parse_simple(self):
         parsed = opener.parse("osfs://foo/bar")
@@ -100,15 +105,21 @@ class TestParse(unittest.TestCase):
 
 
 class TestRegistry(unittest.TestCase):
+    def test_protocols(self):
+        self.assertIsInstance(opener.registry.protocols, list)
+
     def test_registry_protocols(self):
-        # Check registry.protocols list the names of all available entry points
-
-        protocols = [
-            entry_point.name
-            for entry_point in pkg_resources.iter_entry_points("fs.opener")
+        # Check registry.protocols list the names of all available extension
+        extensions = [
+            pkg_resources.EntryPoint("proto1", "mod1"),
+            pkg_resources.EntryPoint("proto2", "mod2"),
         ]
-
-        self.assertEqual(sorted(protocols), sorted(opener.registry.protocols))
+        m = mock.MagicMock(return_value=extensions)
+        with mock.patch.object(
+            sys.modules["pkg_resources"], "iter_entry_points", new=m
+        ):
+            self.assertIn("proto1", opener.registry.protocols)
+            self.assertIn("proto2", opener.registry.protocols)
 
     def test_unknown_protocol(self):
         with self.assertRaises(errors.UnsupportedProtocol):
@@ -160,6 +171,20 @@ class TestRegistry(unittest.TestCase):
                 "could not instantiate opener; some creation error", str(ctx.exception)
             )
 
+    def test_install(self):
+        """Test Registry.install works as a decorator."""
+        registry = Registry()
+        self.assertNotIn("foo", registry.protocols)
+
+        @registry.install
+        class FooOpener(opener.Opener):
+            protocols = ["foo"]
+
+            def open_fs(self, *args, **kwargs):
+                pass
+
+        self.assertIn("foo", registry.protocols)
+
 
 class TestManageFS(unittest.TestCase):
     def test_manage_fs_url(self):
@@ -183,6 +208,12 @@ class TestManageFS(unittest.TestCase):
 
 
 class TestOpeners(unittest.TestCase):
+    def setUp(self):
+        self.tmpdir = tempfile.mkdtemp()
+
+    def tearDown(self):
+        shutil.rmtree(self.tmpdir)
+
     def test_repr(self):
         # Check __repr__ works
         for entry_point in pkg_resources.iter_entry_points("fs.opener"):
@@ -207,10 +238,10 @@ class TestOpeners(unittest.TestCase):
         try:
             # Test creating zip
             with opener.open_fs("zip://" + zip_name, create=True) as make_zip:
-                make_zip.settext("foo.txt", "foofoo")
+                make_zip.writetext("foo.txt", "foofoo")
             # Test opening zip
             with opener.open_fs("zip://" + zip_name, writeable=False) as zip_fs:
-                self.assertEqual(zip_fs.gettext("foo.txt"), "foofoo")
+                self.assertEqual(zip_fs.readtext("foo.txt"), "foofoo")
         finally:
             os.remove(zip_name)
 
@@ -221,10 +252,10 @@ class TestOpeners(unittest.TestCase):
             # Test creating tar
             with opener.open_fs("tar://" + tar_name, create=True) as make_tar:
                 self.assertEqual(make_tar.compression, "gz")
-                make_tar.settext("foo.txt", "foofoo")
+                make_tar.writetext("foo.txt", "foofoo")
             # Test opening tar
             with opener.open_fs("tar://" + tar_name, writeable=False) as tar_fs:
-                self.assertEqual(tar_fs.gettext("foo.txt"), "foofoo")
+                self.assertEqual(tar_fs.readtext("foo.txt"), "foofoo")
         finally:
             os.remove(tar_name)
 
@@ -233,7 +264,10 @@ class TestOpeners(unittest.TestCase):
         mem_fs_2 = opener.open_fs(mem_fs)
         self.assertEqual(mem_fs, mem_fs_2)
 
-    def test_open_userdata(self):
+    @mock.patch("appdirs.{}".format(UserDataFS.app_dir), autospec=True, spec_set=True)
+    def test_open_userdata(self, app_dir):
+        app_dir.return_value = self.tmpdir
+
         with self.assertRaises(errors.OpenerError):
             opener.open_fs("userdata://foo:bar:baz:egg")
 
@@ -242,25 +276,50 @@ class TestOpeners(unittest.TestCase):
         self.assertEqual(app_fs.app_dirs.appauthor, "willmcgugan")
         self.assertEqual(app_fs.app_dirs.version, "1.0")
 
-    def test_open_userdata_no_version(self):
+    @mock.patch("appdirs.{}".format(UserDataFS.app_dir), autospec=True, spec_set=True)
+    def test_open_userdata_no_version(self, app_dir):
+        app_dir.return_value = self.tmpdir
+
         app_fs = opener.open_fs("userdata://fstest:willmcgugan", create=True)
         self.assertEqual(app_fs.app_dirs.appname, "fstest")
         self.assertEqual(app_fs.app_dirs.appauthor, "willmcgugan")
         self.assertEqual(app_fs.app_dirs.version, None)
 
-    def test_user_data_opener(self):
-        user_data_fs = open_fs("userdata://fstest:willmcgugan:1.0")
+    @mock.patch("appdirs.{}".format(UserDataFS.app_dir), autospec=True, spec_set=True)
+    def test_user_data_opener(self, app_dir):
+        app_dir.return_value = self.tmpdir
+
+        user_data_fs = open_fs("userdata://fstest:willmcgugan:1.0", create=True)
         self.assertIsInstance(user_data_fs, UserDataFS)
         user_data_fs.makedir("foo", recreate=True)
-        user_data_fs.settext("foo/bar.txt", "baz")
+        user_data_fs.writetext("foo/bar.txt", "baz")
         user_data_fs_foo_dir = open_fs("userdata://fstest:willmcgugan:1.0/foo/")
-        self.assertEqual(user_data_fs_foo_dir.gettext("bar.txt"), "baz")
+        self.assertEqual(user_data_fs_foo_dir.readtext("bar.txt"), "baz")
 
     @mock.patch("fs.ftpfs.FTPFS")
     def test_open_ftp(self, mock_FTPFS):
         open_fs("ftp://foo:bar@ftp.example.org")
         mock_FTPFS.assert_called_once_with(
-            "ftp.example.org", passwd="bar", port=21, user="foo", proxy=None, timeout=10
+            "ftp.example.org",
+            passwd="bar",
+            port=21,
+            user="foo",
+            proxy=None,
+            timeout=10,
+            tls=False,
+        )
+
+    @mock.patch("fs.ftpfs.FTPFS")
+    def test_open_ftps(self, mock_FTPFS):
+        open_fs("ftps://foo:bar@ftp.example.org")
+        mock_FTPFS.assert_called_once_with(
+            "ftp.example.org",
+            passwd="bar",
+            port=21,
+            user="foo",
+            proxy=None,
+            timeout=10,
+            tls=True,
         )
 
     @mock.patch("fs.ftpfs.FTPFS")
@@ -273,4 +332,5 @@ class TestOpeners(unittest.TestCase):
             user="foo",
             proxy="ftp.proxy.org",
             timeout=10,
+            tls=False,
         )
